package protocolP2P;
import java.util.Arrays;
import protocolP2P.Payload;
import protocolP2P.RequestResponseCode;
import exception.TransmissionError;
import exception.ProtocolError;
import exception.InternalError;
import exception.SizeError;
import java.io.UnsupportedEncodingException;

/** Representation of payload for list response.
 * @author	Louis Royer
 * @author	Flavien Haas
 * @author	JS Auge
 * @version	1.0
 */
public class FileList extends Payload {
	private String[] fileList;

	/** Constructor (typically used by the server) with an ArrayList parameter containing
	 * filenames.
	 * @param fileList	a list of files. Must not be empty.
	 * @throws	InternalError
	 */
	public FileList(String[] fileList) throws InternalError {
		super(RequestResponseCode.LIST_RESPONSE);
		/* assert to help debugging */
		assert fileList.length != 0 : "Payload size of FileList must not be empty, use EmptyDirectory from Payload instead";
		if (fileList.length == 0) {
			throw new InternalError();
		}
		this.fileList = fileList;
	}

	/** Constructor (typically used by client) with a byte[] parameter containing the Packet received.
	 * @param packet	the full packet received
	 * @throws	SizeError
	 * @throws	InternalError
	 * @throws	ProtocolError
	 * @throws	TransmissionError
	 */
	protected FileList(byte[] packet) throws TransmissionError, SizeError, ProtocolError, InternalError {
		super(packet);
		/* assert to help debugging */
		assert requestResponseCode == RequestResponseCode.LIST_RESPONSE : "FileList subclass is incompatible with this Packet, request/response code must be checked before using this constructor";
		/* InternalErrorException */
		if (requestResponseCode!= RequestResponseCode.LIST_RESPONSE) {
			throw new InternalError();
		}
		int size = getPayloadSize(packet);
		try {
			fileList = (new String(packet, PAYLOAD_START_POSITION, size, "UTF-8")).split("\n");
		} catch (UnsupportedEncodingException e) {
			throw new InternalError();
		}
	}

	/** Returns a byte[] containing Packet with padding.
	 * This Packet is still incomplete and should not be send directly.
	 * ProtocolP2PPacket will use this method to generate the complete Packet.
	 * @return Packet with padding
	 * @throws	InternalError
	 */
	protected byte[] toPacket() throws InternalError {
		// compute size
		int size = PAYLOAD_START_POSITION;
		for (String s : fileList) {
			size += s.length();
			size += 1; // size for '\n'
		}
		size -=1; // remove trailing '\n'
		byte[] packet = new byte[size]; // java initialize all to zero
		// set request/response code
		packet[RequestResponseCode.RRCODE_POSITION] = requestResponseCode.codeValue;
		// set payload size
		setPayloadSize(size - PAYLOAD_START_POSITION, packet);
		// Write fileList
		int bCount = PAYLOAD_START_POSITION;
		for(String s : fileList) {
			if (bCount != PAYLOAD_START_POSITION) { // not on first iteration
				try {
					packet[bCount] = "\n".getBytes("UTF-8")[0]; // separator
				} catch (UnsupportedEncodingException e) {
					throw new InternalError();
				}
				bCount += 1;
			}
			// Copy filename
			try {
				byte[] sb = s.getBytes("UTF-8");
				for(byte b : sb) {
					packet[bCount] = b;
					bCount += 1;
				}
			} catch (UnsupportedEncodingException e) {
				throw new InternalError();
			}
			
		}
		return packet;
	}

	/** fileList getter.
	 * @return	fileList
	 */
	public String[] getFileList() {
		return fileList;
	}
}
